##
# Base class for objects
module Tardef
  class Base
    attr_accessor :extensions

    def add_extension(extension)
      @extensions << extension
    end

    def to_hash
      hash = {}
      # Add all instance variables
      instance_variables.each do |var| 
        if not instance_variable_get(var) or instance_variable_get(var).empty?
          next
        end
        hash[var.to_s.delete("@")] = instance_variable_get(var)
      end
      # Tidy up the extensions by making them hashes
      return hash if not hash["extensions"]
      extensions = []
      hash["extensions"].each do |extension|
        extensions << extension.to_hash
      end
      hash["extensions"] = extensions
      hash
    end
  
    def load_hash(hash)
      hash.each do |key, value|
        key = "@" + key
        if instance_variable_defined?(key)
          instance_variable_set(key, value)
        end
      end
      return unless @extensions
      
      extension_list = []
      @extensions.each do |extension|
        ext = Extension.new(nil, nil)
        ext.load_hash(extension)
        extension_list << ext
      end
      @extensions = extension_list
    end
  
    def initialize
      @extensions = []
    end
  end

  class File < Base
    attr_accessor :filename, :title, :editor, :imports, :schema
    attr_accessor :last_changed, :lists, :tags, :tasks, :extension_defs
  
    def initialize(filename, protocol_version, last_changed = Time.now.iso8601(0), title = "", editor = "", imports = [], schema = Tardef::DEFAULT_SCHEMA)
      super()
      @filename, @last_changed, @title = filename, last_changed, title
      @editor, @lists, @tags, @tasks, @imports = editor, [], [], [], imports
      @schema = schema
    end
  
    def add_list(list)
      @lists << list
    end
  
    def add_tag(tag)
      @tags << tag
    end
  
    def add_task(task)
      @tasks << task
    end
      
    def has_list?(uid)
      hash_has_object?(@lists, uid)
    end
  
    def has_tag?(tag)
      hash_has_object?(@tags, tag)
    end 

    def has_task?(task)
      hash_has_object?(@tasks, task)
    end 
    
    def to_hash
      hash = {"meta" => {
        "title" => @title, "editor" => @editor, "last_changed" => @last_changed,
        "imports" => @imports
      },
      "lists" => @lists, "tags" => @tags, "tasks" => @tasks, "schema" => @schema,
      "extensions" => @extensions,
      }
    
      items = []    
      ["lists", "tags", "tasks", "extensions"].each do |type|
        next if not hash[type]
        items = []
        hash[type].each do |item|
          items << item.to_hash
        end
        hash[type] = items 
      end
      return hash
    end
    
    private
    def hash_has_object?(list, uid)
      list.each { |item| return true if item.uid == uid }
      return false
    end
    
  end
end